/***********************************************************************

    This source code is provided as is and may be used to extend 
    the functionality if desired. Do not remove all refernces to the
    original author.

    Ken Schirmeister
    KenSch@MountainCable.net  

***********************************************************************/

// CreateMacro.cpp: implementation of the CCreateMacro class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DatePrinter.h"
#include "CreateMacro.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCreateMacro::CCreateMacro()
{

}

CCreateMacro::~CCreateMacro()
{

}

/////////////////////////////////////////////////////////////////////////////

ushort __stdcall CCreateMacro::ConvertToMegaCadBlock( // R: Anzahl Zeilen
        const CString &strText,
        TCHAR   **ppszBlock,                     // wird allokiert
        bool    bAddMagic /* = false */)         // true --> Zeile "Magic: 42734571" vorweg
{
    // das Editcontrol gibt die Zeilen mit \a\n getrennt, MegaCAD will
    // sie mit '\0' getrennt haben, und braucht die Anzahl der Zeilen:
    ushort iLineCnt = 0;
    *ppszBlock   = NULL;
    int iDst = 0;

    if ( strText.GetLength() )
    {
        CString strMagic;
        if ( bAddMagic )
            strMagic.Format("Magic: %d", MY_MAGIC);

        *ppszBlock = new TCHAR[strText.GetLength() + 1 + strMagic.GetLength() + 1];
        if ( bAddMagic )
        {
            strcpy(*ppszBlock, strMagic);
            iDst += strMagic.GetLength() + 1;  // hinter das '\0'
            iLineCnt++;
        }

        for ( int i = 0 ; i < strText.GetLength() ; i++ )
        {
            if ( strText[i] == '\r' )
                ;
            else if ( strText[i] == '\n' )
            {
                (*ppszBlock)[iDst++] = '\0';
                iLineCnt++;
            }
            else
                (*ppszBlock)[iDst++] = strText[i];
        }
        // Wenn das letzte Zeichen nicht '\n' ist, so haben wir eine Zeile mehr:
        if ( strText[strText.GetLength() - 1] != '\n' )
        {
            (*ppszBlock)[iDst] = '\0';
            iLineCnt++;
        }
    }
    return iLineCnt;
}
    
/////////////////////////////////////////////////////////////////////////////

ushort __stdcall CCreateMacro::CreateMegaCadBlocks( // R: Anzahl Zeilen
        const CString &strDbInfo,
        TCHAR   **ppszBlockText,              // wird allokiert
        TCHAR   **ppszBlockDbInfo)            // wird allokiert
{
    ushort iCntLines;
    CString strTextBlock;

    m_Interpreter.Interpret(strDbInfo, &strTextBlock);

    if ( iCntLines = ConvertToMegaCadBlock(strTextBlock, ppszBlockText) )
        VERIFY (    ConvertToMegaCadBlock(strDbInfo,    ppszBlockDbInfo, true) == iCntLines + 1);

    return iCntLines;
}

/////////////////////////////////////////////////////////////////////////////

void __stdcall CCreateMacro::CreateMacro(
        const CString &strText)
{
    t_text  txtData;
    t_macro      mac;           // macro data
    t_mkmacro    mkm;           // entities to be created
    t_placemac   placemac;      // how and where to place the macro


    // Create a macro
    memset(&txtData,  0, sizeof(txtData));
    memset(&mac,      0, sizeof(mac));
    memset(&mkm,      0, sizeof(mkm));
    memset(&placemac, 0, sizeof(placemac));

    // die in MegaCAD eingestellten Textdaten holen
    GetTextVal(E_TXTBLK,TRUE, &txtData);

    txtData.numl = CreateMegaCadBlocks(strText, &txtData.str, &mac.info);
    mac.numl = (ushort)(txtData.numl + 1);            // wegen Magic:

    mac.name = "??DatePrinter??.mac";
    mkm.data = &txtData;
    mkm.type = E_TXTBLK;

    ::CreateMacro(&mac, &mkm, 1, &placemac, NULL, 0);

    delete[] txtData.str;
    delete[] mac.info;
}



