/***********************************************************************

    This source code is provided as is and may be used to extend 
    the functionality if desired. Do not remove all refernces to the
    original author.

    Ken Schirmeister
    KenSch@MountainCable.net  

***********************************************************************/



// DlgMain.cpp : implementation file
//

#include "stdafx.h"
#include "TrivialTools.h"
#include "DatePrinter.h"
#include "MacroData.h"
#include "Interpreter.h"
#include "DlgMain.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////

static Formats_t aDateFormats[] =
{
    { "Date-Locale-Long",  "%#x"} ,
    { "Date-Locale-Short", "%x"} ,
    { "DD.MM.YYYY", "%d.%m.%Y"} ,
    { "MM.DD.YYYY", "%m.%d.%Y"} ,
    { "YYYY.MM.DD", "%Y.%m.%d"} ,
};

static Formats_t aTimeFormats[] =
{
    { "Time-Locale", "%X"     } ,
    { "15:30",   "%H:%M"      } ,
    { "3:30 pm", "%I:%M %p"   } ,
};                          

static Formats_t aFileFormats[] =
{
    { "Short", "%s"     } ,
    { "Long",  "%FN"     } ,
};                          

static Formats_t aNameFormats[] =
{
    { "Text", "%UN"     } ,
};                          

/////////////////////////////////////////////////////////////////////////////
// CDlgMain dialog


CDlgMain::CDlgMain(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgMain::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgMain)
	m_strRichEditText = _T("");
	m_strRichEditPreview = _T("");
	//}}AFX_DATA_INIT
    m_iFieldSelection  = -1;
    m_iFormatSelection = -1;
    m_pCurrentFormat   = NULL;
}

/////////////////////////////////////////////////////////////////////////////

/*  eine sehr simple und nicht beseonsers ausgeklgelte Mglichkeit die
    Texte im Dialog bersetzbar zu machen. Wemm die Datei
        "DatePrinter.txt"
    in aktuellen Verzeichnis (d.h. im MegaCAD-Verzechnis) exisiert, 
    so werden die Texte eingelesen. Wenn nicht, dann bleiben die
    englischen halt stehen.
*/

void __stdcall CDlgMain::TranslateTexts()
{
    FILE *fp;
    TCHAR szText[128];
    if ( fp = fopen("DatePrinter.txt", "rt") )
    {
        if ( ReadString(fp, szText, numof(szText)) )
            SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDC_STATIC_FIELD_NAME)->SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDC_STATIC_FORMAT)->SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDC_STATIC_EDIT_HERE)->SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDC_STATIC_PREVIEW)->SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDOK)->SetWindowText(szText);
        if ( ReadString(fp, szText, numof(szText)) )
            GetDlgItem(IDCANCEL)->SetWindowText(szText);

        CMacroData::ReadTexts(fp);

        ReadString(fp, aDateFormats[0].szShowInDialog, numof(aDateFormats[0].szShowInDialog));
        ReadString(fp, aDateFormats[1].szShowInDialog, numof(aDateFormats[1].szShowInDialog));
        ReadString(fp, aTimeFormats[0].szShowInDialog, numof(aTimeFormats[0].szShowInDialog));
        ReadString(fp, aFileFormats[0].szShowInDialog, numof(aFileFormats[0].szShowInDialog));
        ReadString(fp, aFileFormats[1].szShowInDialog, numof(aFileFormats[1].szShowInDialog));
        fclose(fp);
    }
}

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgMain)
	DDX_Control(pDX, IDC_RICHEDIT_PREVIEW, m_ctrlRichEditPreview);
	DDX_Control(pDX, IDC_RICHEDIT_TEXT, m_ctrlRichEditText);
	DDX_Control(pDX, IDC_LIST_FORMATS, m_ctrlListFormats);
	DDX_Control(pDX, IDC_LIST_FIELDS, m_ctrlListFields);
	DDX_Text(pDX, IDC_RICHEDIT_TEXT, m_strRichEditText);
	DDX_Text(pDX, IDC_RICHEDIT_PREVIEW, m_strRichEditPreview);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgMain, CDialog)
	//{{AFX_MSG_MAP(CDlgMain)
	ON_LBN_SELCHANGE(IDC_LIST_FIELDS, OnSelchangeListFields)
	ON_LBN_SELCHANGE(IDC_LIST_FORMATS, OnSelchangeListFormats)
	ON_BN_CLICKED(IDC_BUTTON_INSERT, OnButtonInsert)
	ON_EN_CHANGE(IDC_RICHEDIT_TEXT, OnChangeRicheditText)
	ON_LBN_DBLCLK(IDC_LIST_FORMATS, OnDblclkListFormats)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

void __stdcall CDlgMain::CheckControls()
{
    int iNewField = m_ctrlListFields.GetCurSel();
    int i;
    if ( m_iFieldSelection != iNewField )
    {
        Field_t *pFieldSelected = &CMacroData::sm_aMacros[iNewField];
        while ( m_ctrlListFormats.DeleteString(0) != LB_ERR )
            ;
        switch (pFieldSelected->iType )
        {
            case Date:
                for ( i = 0 ; i < numof(aDateFormats) ; i++ )
                    m_ctrlListFormats.AddString(aDateFormats[i].szShowInDialog);
                break;
            case Time:
                for ( i = 0 ; i < numof(aTimeFormats) ; i++ )
                    m_ctrlListFormats.AddString(aTimeFormats[i].szShowInDialog);
                break;
            case FileName:
                for ( i = 0 ; i < numof(aFileFormats) ; i++ )
                    m_ctrlListFormats.AddString(aFileFormats[i].szShowInDialog);
                break;
            case UserName:
                for ( i = 0 ; i < numof(aNameFormats) ; i++ )
                    m_ctrlListFormats.AddString(aNameFormats[i].szShowInDialog);
                break;
        }
        m_ctrlListFormats.SetCurSel(0);
        m_iFieldSelection = iNewField;
    }

    TCHAR szExample[256];
    switch ( CMacroData::sm_aMacros[m_iFieldSelection].iType )
    {
        case Date:
            m_pCurrentFormat = &aDateFormats[m_ctrlListFormats.GetCurSel()];
            break;
        case Time:
            m_pCurrentFormat = &aTimeFormats[m_ctrlListFormats.GetCurSel()];
            break;
        case FileName:
            m_pCurrentFormat = &aFileFormats[m_ctrlListFormats.GetCurSel()];
            break;
        case UserName:
            m_pCurrentFormat = &aNameFormats[m_ctrlListFormats.GetCurSel()];
            break;
    }

    m_TimeGetter.GetTime(
                    CMacroData::sm_aMacros[m_iFieldSelection].iTimeType,
                    m_pCurrentFormat->pszFormat,
                    m_szFileName,
                    szExample);

    GetDlgItem(IDC_EDIT_FIELD_PREVIEW)->SetWindowText(szExample);
}

/////////////////////////////////////////////////////////////////////////////

void __stdcall CDlgMain::UpdatePreview()
{
    UpdateData(true);
    m_strRichEditPreview.Empty();
    m_Interpreter.Interpret(m_strRichEditText, &m_strRichEditPreview);
    GetDlgItem(IDOK)->EnableWindow(!m_strRichEditText.IsEmpty());
    UpdateData(false);
}

/////////////////////////////////////////////////////////////////////////////

BOOL CDlgMain::OnInitDialog() 
{
	CDialog::OnInitDialog();

    TranslateTexts();
	
    for ( int i = 0 ; i < CMacroData::sm_iCnt ; i++ )
        m_ctrlListFields.AddString(CMacroData::sm_aMacros[i].szField);

    m_ctrlListFields.SetCurSel(0);
    CheckControls();
    UpdatePreview();

    m_ctrlRichEditText.SendMessage(EM_SETEVENTMASK, 0, ENM_CHANGE);

    	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::OnSelchangeListFields() 
{
    CheckControls();
}

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::OnSelchangeListFormats() 
{
    CheckControls();
}

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::OnDblclkListFormats() 
{
    OnButtonInsert();	
}

/////////////////////////////////////////////////////////////////////////////
// CDlgMain message handlers

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::OnChangeRicheditText() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
    UpdatePreview();
}

/////////////////////////////////////////////////////////////////////////////

void CDlgMain::OnButtonInsert() 
{
    ASSERT(m_pCurrentFormat);
    TCHAR szText[128];
    Field_t *p = &CMacroData::sm_aMacros[m_iFieldSelection];

    sprintf(szText, "$(*%s %s*)", p->pszMacroText, m_pCurrentFormat->pszFormat); 
    m_ctrlRichEditText.ReplaceSel(szText, true);
    UpdatePreview();
    m_ctrlRichEditText.SetFocus();
}

/////////////////////////////////////////////////////////////////////////////

bool __stdcall CDlgMain::DoDialog(
        CString *pstrText)
{
    bool bOK;
    CDlgMain dlg;
    dlg.m_strText = *pstrText;
    GetCurrentFile((uchar *)dlg.m_szFileName);
    if ( bOK = (dlg.DoModal() == IDOK) )
    {
        *pstrText = dlg.m_strRichEditText;
    }
    return bOK;
}

