/***********************************************************************

    This source code is provided as is and may be used to extend 
    the functionality if desired. Do not remove all refernces to the
    original author.

    Ken Schirmeister
    KenSch@MountainCable.net  

***********************************************************************/

// GetTime.cpp: implementation of the CGetTime class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "DatePrinter.h"
#include "GetTime.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGetTime::CGetTime()
{
    _tzset();
}

//////////////////////////////////////////////////////////////////////

CGetTime::~CGetTime()
{

}

//////////////////////////////////////////////////////////////////////

TCHAR * __fastcall CGetTime::GetTime(
        TimeType_t iType,
        const TCHAR *pszTimeFormat,
        const TCHAR *pszFileName,     // ignored when iType == Current && != Name
        TCHAR szTimeString[128])
{
    TCHAR *pRet = szTimeString;
    struct tm *pTime;
    time_t ltime;

    if ( iType == Name )
    {
        if ( strstr(pszTimeFormat, "%UN") )  // Current User:
        {
            ulong iCnt = 127;
            if ( GetUserName(szTimeString, &iCnt) == 0 )
                strcpy(szTimeString, "Unknown User");
        }
        else
        {
            if ( !pszFileName || pszFileName[0] == '\0' )
                pszFileName = "*UNNAMED*";
            if ( strstr(pszTimeFormat, "%FN") )
                strcpy(szTimeString, pszFileName);
            else
            {
                const TCHAR *pos;
                if ( (pos = strrchr(pszFileName, '\\')) ||
                     (pos = strrchr(pszFileName, '/') ) ||
                     (pos = strrchr(pszFileName, ':') ) )
                {
                    strcpy(szTimeString, pos + 1);
                }
                else
                    strcpy(szTimeString, pszFileName);
            }
        }
    }
    else
    {
        time( &ltime );

        ASSERT(iType == Current || pszFileName);

        struct _stat stats;

        if ( iType != Current )
        {
            if ( _stat(pszFileName, &stats) == 0 )
            {
                switch ( iType )
                {
                    case Created:
                        ltime = stats.st_ctime;
                        break;
                    case Modified:
                        ltime = stats.st_mtime;
                        break;
                    case Accessed:
                        ltime = stats.st_atime;
                        break;
                }
            }
        }
        pTime = localtime(&ltime);

        strftime( szTimeString, 128, pszTimeFormat, pTime);
    }
    return pRet;
}
